# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

def parent_dirpath(fdpath: str): ...

def fmt_seconds(t1, t2=None): ...

def fmt_bytes(num: int): ...

def fmt_file_size(filepath: str): ...

def timestamp(ts): ...

def path2name(path_str: str) -> str: ...



# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python
from __future__ import division
from __future__ import print_function

from cProfile import Profile
from functools import wraps
from inspect import getfile
from os import chdir
from os import getcwd
from os import makedirs
from os import path, mkdir
from time import time
from typing import Callable
from typing import Tuple
from pupy.utils import fmt_seconds
import logging
from logging.config import dictConfig

# "%(color)s[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d]%(end_color)s %(message)s"
logging_config = dict(
    version=1,
    formatters={
        'f': {
            'format': "[%(levelname)1.1s %(asctime)s %(filename)s:%(lineno)d] %(message)s"
            }
        },
    handlers={
        'h': {
            'class'    : 'logging.StreamHandler',
            'formatter': 'f',
            'level'    : logging.DEBUG
            }
        },
    root={
        'handlers': ['h'],
        'level'   : logging.DEBUG,
        },
    )

dictConfig(logging_config)

logger = logging.getLogger()

def in_n_out(funk: Callable):
    """Chdir in to the dir the test_function is in and change dirs out when done

    :type funk: Callable
    :param funk: docin.api logger functions logger.(debug/info/warn/error)
    :return: wrapped function
    """

    @wraps(funk)
    def chin_n_chout(*args, **kwargs):
        """

        :param args:
        :param kwargs:
        :return:
        """
        # thing = os.path.split(os.path.realpath(__file__))
        cd = getcwd()
        dirpath = args[0]
        if path.isdir(dirpath):
            chdir(dirpath)
        funk_results = funk(*args, **kwargs)
        chdir(cd)
        return funk_results

    return chin_n_chout

def flog(funk=None, loglevel='debug', funk_call=True, tictoc=False):
    D = {
        'debug': logger.debug,
        'info' : logger.info,
        'warn' : logger.warning,
        'error': logger.error
        }

    def _decorate_flog_wrapper(_funk):
        def _fmt_args(*args):
            return ", ".join(str(arg) for arg in args)

        def _fmt_kwargs(**kwargs):
            return ", ".join("{}={}".format(str(k), str(v))
                             for k, v in kwargs.items())

        def _fmt_call(*args, **kwargs):
            params_str = ", ".join(
                s for s in (_fmt_args(*args), _fmt_kwargs(**kwargs)) if s
                )
            return "{}({})".format(_funk.__name__, params_str)

        @wraps(_funk)
        def _flog_wrapper(*args, **kwargs):
            ti = time()
            _ret = _funk(*args, **kwargs)
            tf = time()
            msg_parts = [_fmt_call(*args, **kwargs) if funk_call else None,
                         fmt_seconds(ti, tf) if tictoc else None]
            msg_str = ' | '.join(part for part in msg_parts if part)
            if any(el for el in msg_parts):
                D[loglevel]("[FLOG] | {}".format(msg_str))
            return _ret

        return _flog_wrapper

    return _decorate_flog_wrapper(funk) if funk else _decorate_flog_wrapper

def dirdec(funk):
    @wraps(funk)
    def _wrapper(*args, **kwargs):
        result = funk(*args, **kwargs)
        try:
            mkdir(result)
        except (FileExistsError, OSError) as e:
            pass
        return result

    return _wrapper

def mkdirs(funk):
    @wraps(funk)
    def _wrapper(*args, **kwargs):
        dirpath = path.split(args[0])[0]
        try:
            makedirs(dirpath, exist_ok=True)
        except OSError:
            pass
        return funk(*args, **kwargs)

    return _wrapper

def cash_it(funk):
    """args-2-return value cache.
    
    This function is particularly useful for when you want that lru-cache,
    but you/one is working with python two.

    :param funk: function to be cached
    :type funk: function
    :returns: wrapped function
    :rtype: function

    """
    cash_money = {}

    @wraps(funk)
    def cash_wrap(*argz):
        """

        """
        if argz not in cash_money:
            rv = funk(*argz)
            cash_money[argz] = rv
        return cash_money[argz]

    return cash_wrap

def cprof(funk):
    """"cProfiling decorator
    
    src: https://zapier.com/engineering/profiling-python-boss/

    :param funk: 

    """

    @wraps(funk)
    def profiled_funk(*args, **kwargs):
        """

        """
        profile = Profile()
        try:
            profile.enable()
            ret_val = funk(*args, **kwargs)
            profile.disable()
        finally:
            print("__CPROFILE__")
            profile.print_stats()
        return ret_val

    return profiled_funk

class tictoc(object):
    """Timing decorator object

    :param runs: # of runs to time over (defaults to 1)

    """

    def __init__(self, runs=1):
        self.runs = runs

    def __str__(self, t_total, funk, args_string):
        str_list = [
            "__TICTOC__",
            "    file: {}".format(getfile(funk)),
            "    funk: {}".format(funk.__name__),
            "    args: {}".format(args_string),
            "    time: {}".format(tictoc.ftime(t_total)),
            "    runs: {}".format(self.runs),
            ]
        return "\n".join(str_list)

    def __call__(self, time_funk, printing=True):
        @wraps(time_funk)
        def time_wrapper(*args, **kwargs):
            """

            """
            self.args = str(args)
            ts = time()
            for i in range(self.runs):
                result = time_funk(*args, **kwargs)
            te = time()
            t_total = (te - ts) / self.runs
            if printing:
                print(self.__str__(t_total, time_funk, str(args)))
            return result

        return time_wrapper

    @staticmethod
    def ftime(t1, t2=None):
        """Formats time string
        
        Formats t1 if t2 is None as a string; Calculates the time and formats
        the time t2-t1 if t2 is not None.

        :param t1: time 1
        :type t1: double
        :param t2: time 2 (Default value = None)
        :type t2: None or double
        :returns: -> formated time string
        :rtype: str

        """
        if t2 is not None:
            return tictoc.ftime((t2 - t1))
        elif t1 == 0.0:
            return "~0.0~"
        elif t1 >= 1:
            return "%.3f s" % t1
        elif 1 > t1 >= 0.001:
            return "%.3f ms" % ((10 ** 3) * t1)
        elif 0.001 > t1 >= 0.000001:
            return "%.3f μs" % ((10 ** 6) * t1)
        elif 0.000001 > t1 >= 0.000000001:
            return "%.3f ns" % ((10 ** 9) * t1)
        else:
            return tictoc.ftime((t2 - t1))



# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python
from typing import Any
from typing import Generator
from typing import Iterable
from typing import List
from typing import Sequence
from typing import Tuple
from typing import Union

Flint = Union[int, float]

def partitions_gen(numero: int, min_p: int = ..., max_p: int = ...) -> Generator: ...
def rfactorial(n: int): ...
def radians_2_degrees(rads: Flint): ...
def degrees_2_radians(degs: Flint): ...
def power_mod(number: Flint, exponent: Flint, mod: Flint): ...
def divisors_gen(n: int) -> Generator: ...
def gcd_it(a: int, b: int): ...
def gcd_r(a: int, b: int): ...
def reverse(n: int): ...
def fib_r(n: int): ...
def expo(d: Any, n: Any): ...
def pytriple_gen(max_c: int) -> Generator: ...
def repermutations(toop: tuple) -> int: ...
def disjoint(a: Iterable, b: Iterable) -> bool: ...
def n_choose_r(n: int, r: int) -> int: ...
def pytriple_gen_2() -> None: ...
def get_pythag_triple(real: Any, imag: Any): ...
def set_cmp(a: Iterable, b: Iterable) -> Tuple[set, set, set]: ...

class Trigon:
    pt1: Any = ...
    pt2: Any = ...
    pt3: Any = ...
    def __init__(self, pt1: Any, pt2: Any, pt3: Any) -> None: ...
    @classmethod
    def from_points(cls, pts: List[Any]): ...
    def __str__(self): ...
    def __contains__(self, point: Any): ...
    def inner_triangles(self, point: Any): ...
    def is_perimeter_point(self, point: Any): ...
    def points(self): ...
    def contains_origin(self): ...
    def area(self): ...
    @staticmethod
    def area_from_points(pt1: Any, pt2: Any, pt3: Any): ...

class Vuple(tuple):
    Voops = Union[tuple, int, float, Sequence]
    def __new__(cls, *args: Any): ...
    def __gt__(self, other: Any): ...
    def __eq__(self, other: Any): ...
    def __add__(self, k: Voops): ...
    def __sub__(self, k: Voops): ...
    def __mul__(self, k: Union[tuple, int, float]): ...
    def __imul__(self, k: Union[tuple, int, float]): ...
    def _mul_scalar(self, k: Flint): ...
    def __truediv__(self, k: Voops): ...
    def _truediv_scalar(self, k: Flint): ...
    def __itruediv__(self, k: Voops): ...
    def __floordiv__(self, k: Voops): ...
    def __ifloordiv__(self, k: Voops): ...
    def _floordiv_scalar_int(self, k: Flint): ...
    def normalize(self): ...
    def get_mag_sqrd(self): ...
    def get_mag(self): ...
    def is_disjoint(self, them: Any): ...
    def product(self): ...
    @staticmethod
    def unit_vuple(voop: Union[Vuple, tuple]): ...
    @staticmethod
    def mag_sqrd(voop: Union[Vuple, tuple, List[Union[int, float]]]): ...
    @staticmethod
    def mag(voop: Any): ...
    @staticmethod
    def dot(a: Any, b: Any): ...
    @staticmethod
    def cross(v1: Any, v2: Any): ...
    @staticmethod
    def angle(
        v1: Union[Vuple, tuple], v2: Union[Vuple, tuple], radians: bool = ...
    ): ...



# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python
from typing import Any
from typing import Callable
from typing import Dict
from typing import Optional

def cash_it(funk: Callable): ...
def mkdirs(funk: Callable): ...
def dirdec(funk: Callable): ...
def flog(funk: Callable): ...
def in_n_out(funk: Callable): ...

class Jasm:
    file_path: str = ...
    def __init__(self, path: str) -> None: ...
    def __call__(self, funk: Callable): ...
    @staticmethod
    def read(fpath: str): ...
    @staticmethod
    def write(fpath: str, obj: Dict) -> None: ...

def cprof(funk: Callable): ...

class tictoc(object):
    runs: Any = ...
    def __init__(self, runs: int = ...) -> None: ...
    args: Any = ...
    def __call__(self, time_funk: Callable, printing: bool = ...): ...
    @staticmethod
    def ftime(t1: float, t2: Optional[float] = ...): ...



# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

from __future__ import print_function
from __future__ import with_statement

from codecs import getwriter
from io import open
from itertools import count
from os import path
from os import utime

from pupy.decorations import mkdirs

try:
    from ujson import dump
    from ujson import load
except Exception as e:
    from json import dump
    from json import load

def safepath(path_str):
    """Checks if a file/dir path is save/unused; returns an unused path.

    :param path_str:

    """
    if path.exists(path_str):
        f_bn, f_ext = path.splitext(path_str)
        for n in count(1):
            safe_save_path = f_bn + "_({}).".format(str(n)) + f_ext
            if not path.exists(safe_save_path):
                return safe_save_path
    return path_str

@mkdirs
def savings(filepath, string, clobber=True):
    """Writes a string to filepath


    :param filepath: Filepath save location
    :param string: File as a string to be saved
    :param clobber: Save over a file if the filepath exists
    :return: None? what do you want? confirmation?

    .. note:: Decorated w/ @mkdirs
        Function is decorated with @mkdirs decorator; @mkdirs creates parent
        directories for the given filepath if they do not already exist.

    """
    if not clobber and path.exists(filepath):
        filepath = safepath(filepath)
    with open(filepath, "wb") as file:
        file.write(string.encode("utf-8"))

def loads(filepath):
    """Read and return the file-contents as a string given a filepath

    :param filepath: Path to a file to read
    :return: Content of the file read as a string

    """
    try:
        with open(filepath, "r", encoding="utf-8") as f:
            return f.read()
    except UnicodeDecodeError as e:
        with open(filepath, "r", encoding="latin2") as f:
            return f.read()

@mkdirs
def sjson(filepath, data, min=False):
    """Save json-serial-ize-able data to a specific filepath.

    :param filepath: destination filepath
    :param data: json cereal-izable dictionary/list/thing
    :param min: Bool flag -- minify the json file
    :return: None

    """
    if type(data) == dict and any(type(val) == bytes for val in data.values()):
        data = {k: str(v, encoding="utf-8") for k, v in data.items()}
    with open(filepath, "wb") as jsonfile:
        if min:
            dump(data, getwriter("utf-8")(jsonfile), ensure_ascii=False)
        else:
            dump(
                data,
                getwriter("utf-8")(jsonfile),
                indent=4,
                sort_keys=True,
                ensure_ascii=False,
                )

def save_jasm(filepath, data, min=False):
    """Alias for sjson (which stands for 'save-json')"""
    return sjson(filepath, data, min)

def sjasm(filepath, data, min=False):
    """Alias for sjson (which stands for 'save-json')"""
    return sjson(filepath, data, min)

def spak(filepath, data, min=False):
    """Alias for sjson (which stands for 'save-json')"""
    return sjson(filepath, data, min)

@mkdirs
def ljson(filepath):
    """Load a json file given a filepath and return the file-data

    :param filepath: path to the jasm file you want to load
    :return: Loaded file contents
    """

    with open(filepath) as infile:
        return load(infile)

def load_jasm(filepath):
    """Alias for ljson (which stands for 'load-json')"""
    return ljson(filepath)

def ljasm(filepath):
    """Alias for ljson (which stands for 'load-json')"""
    return ljson(filepath)

def lpak(filepath):
    """Alias for ljson (which stands for 'load-json')"""
    return ljson(filepath)

@mkdirs
def touch(filepath):
    """Touches a file just like touch on the command line

    :param filepath: filepath to 'touch' in a unix-y sense
    :return: None
    """
    with open(filepath, "a"):
        utime(filepath, None)



# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python
"""
========
Pupy CLI
========
"""
from argparse import ZERO_OR_MORE
from argparse import ArgumentParser
from codecs import decode
from os import listdir
from os import rename

from pupy._version import __version__

def unescaped_str(arg_str):
    return decode(str(arg_str), "unicode_escape")

parser = ArgumentParser(description="Command description.")
parser.add_argument(
    "-r", "--replace", metavar="PAT", nargs=2, help="Rename all things in."
    )
parser.add_argument("-V", "--version", action="store_true", help="Print pupy version.")
parser.add_argument(
    "--rm-pattern",
    metavar="PAT",
    type=unescaped_str,
    nargs=ZERO_OR_MORE,
    help="REPLACE pattern with a space.",
    )

def main(args=None):
    """

    :param args:
    """

    args = parser.parse_args(args=args)
    if args.version:
        print("Pupy version: {}".format(__version__))
    if args.replace:
        frum, two = args.replace
        print("Replacing:", frum)
        print("With:", two)
        for f in listdir("."):
            print("For f/d:", f)
            rename(f, f.replace(frum, two))

    if args.rm_pattern:
        for pattern in args.rm_pattern:
            print("Removing pattern:", pattern)
            for f in listdir("."):
                print("For f/d:", f)
                rename(f, f.replace(pattern, ""))



# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

from os import path, stat
from datetime import datetime

def fmt_bytes(num):
    """
    this function will convert bytes to MB.... GB... etc

    .. doctest:: python

        >>> fmt_bytes(100)
        '100.0 bytes'
        >>> fmt_bytes(1000)
        '1000.0 bytes'
        >>> fmt_bytes(10000)
        '9.8 KB'
        >>> fmt_bytes(100000)
        '97.7 KB'
        >>> fmt_bytes(1000000)
        '976.6 KB'
        >>> fmt_bytes(10000000)
        '9.5 MB'
        >>> fmt_bytes(100000000)
        '95.4 MB'
        >>> fmt_bytes(1000000000)
        '953.7 MB'
        >>> fmt_bytes(10000000000)
        '9.3 GB'
        >>> fmt_bytes(100000000000)
        '93.1 GB'
        >>> fmt_bytes(1000000000000)
        '931.3 GB'
        >>> fmt_bytes(10000000000000)
        '9.1 TB'
        >>> fmt_bytes(100000000000000)
        '90.9 TB'

    """
    for x in ["bytes", "KB", "MB", "GB", "TB"]:
        if num < 1024.0:
            return "%3.1f %s" % (num, x)
        num /= 1024.0

def fmt_file_size(filepath):
    """
    this function will return the file size
    """
    if path.isfile(filepath):
        file_info = stat(filepath)
        return fmt_bytes(file_info.st_size)

def fmt_seconds(t1, t2=None):
    """Formats time string

    Formats t1 if t2 is None as a string; Calculates the time and formats
    the time t2-t1 if t2 is not None.

    :param t1: time 1/initial in seconds
    :type t1: double
    :param t2: time 2 (Default value = None)
    :type t2: None or double
    :returns: formated string of the t1 - t2 or t1
    :rtype: str

    """
    if t2 is not None:
        return fmt_seconds((t2 - t1))
    elif t1 == 0.0:
        return "0 sec"
    elif t1 >= 1:
        return "%.3f sec" % t1
    elif 1 > t1 >= 0.001:
        return "%.3f ms" % ((10 ** 3) * t1)
    elif 0.001 > t1 >= 0.000001:
        return "%.3f μs" % ((10 ** 6) * t1)
    elif 0.000001 > t1 >= 0.000000001:
        return "%.3f ns" % ((10 ** 9) * t1)
    else:
        return fmt_seconds((t2 - t1))

def path2name(path_str):
    """Get the parent-directory for a file or directory path as a string

    :param somepath: path as a string which ya would like the parent of
    :return: The parent directory as a string

    .. doctest:: python

        >>> from os import getcwd
        >>> path2name(getcwd()) in getcwd()
        True

    """
    return path.split(path.abspath(path_str))[-1]

def parent_dirpath(fdpath):
    """

    :param fdpath: file/dir-path as as string
    :return:

    .. doctest:: python

        >>> from os import path
        >>> parent_dirpath(path.abspath(__file__)) in path.abspath(__file__)
        True

    """
    return path.split(fdpath)[0]

def timestamp(ts=None):
    """Time stamp string w/ format yyyymmdd-HHMMSS

    :return: timestamp string

    .. doctest:: python

        >>> from datetime import datetime
        >>> stamps = ['20190225-161151', '20190225-081151']
        >>> timestamp(1551111111.111111) in stamps
        True
        >>> datetime.now().strftime("%Y%m%d-%H%M%S") == timestamp()
        True

    """
    if ts is None:
        return datetime.now().strftime("%Y%m%d-%H%M%S")
    elif isinstance(ts, float):
        return datetime.fromtimestamp(ts).strftime("%Y%m%d-%H%M%S")
    elif isinstance(ts, datetime):
        return ts.strftime("%Y%m%d-%H%M%S")



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['pupy']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pupy',
    'version': '2.4.3',
    'description': 'pretty useful python',
    'long_description': None,
    'author': 'jessekrubin',
    'author_email': 'jessekrubin@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python
from sys import stdout

def yesno(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".
    """
    valid = {
        "yes": True,
        "y"  : True,
        "ye" : True,
        "no" : False,
        "n"  : False
        }
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        out_string = question + prompt
        n_dashes = (
            len(question)
            if "\n" not in question
            else len(question.split("\n")[0])
        )
        out_string = "{}\n{}".format("_" * n_dashes, out_string)
        stdout.write(out_string)
        choice = input().lower()
        if default is not None and choice == "":
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            stdout.write(
                "Please respond with 'yes' or 'no' " "(or 'y' or 'n').\n"
                )



# -*- coding: utf-8 -*-
# ~ Jesse K. Rubin ~ Pretty Useful Python

from typing import Any

def timestamp(): ...
def safepath(filepath: Any): ...
def ensure_save(filepath: Any, n: int = ...): ...
def savings(filepath: Any, string: Any, clobber: bool = ...) -> None: ...
def loads(filepath: Any): ...
def sjson(filepath: Any, data: Any, min: bool = ...) -> None: ...
def save_jasm(filepath: Any, data: Any, min: bool = ...): ...
def sjasm(filepath: Any, data: Any, min: bool = ...): ...
def spak(filepath: Any, data: Any, min: bool = ...): ...
def ljson(filepath: Any): ...
def load_jasm(filepath: Any): ...
def ljasm(filepath: Any): ...
def lpak(filepath: Any): ...
def touch(filepath: Any) -> None: ...
